/* mol.h */
/* Physical parameters of Molecule */
/* 93/01/09 external declinations */
/* 93/10/28 VMAX and JMAX are included in MOLSTRUCT */
/*          definition of VMAX and JMAX are changed */
/*          they represent number of the level but not max level */

struct MOLSTRUCT {
   char name[11];    /* name of the molecule */

   int NV;           /* number of vibrational level */
   int NJ;           /* number of rotational level */
   int LV;           /* nulber of total level VMAX*JMAX-1 */

   int *g;           /* degeneracy */
   double *e;        /* energy of each level */
   double **de;      /* energy difference between each levels */
   double **nyu0;    /* frequency between each levels */
   double **a;       /* Einstein coefficients */
   double **ct;      /* collision rate between each levels */

   double nmol;      /* total amount of molecle [cm-3] */


   double *ng0;      /* initial energy population */
   double *ng;       /* energy population */
};
typedef struct MOLSTRUCT MOLECULE;

/* ----------------------- macro functions ------------------------ */
/* ====== excitation temperature ===== */
#define Tex(mol, u, l) (-(mol.e[u] - mol.e[l])/(WK*log(mol.ng[u]/mol.ng[l])))

/* ---------------- Following Spacies are available now----------------- */
/* =========== in molco.c ============== */
extern MOLECULE CO;        /* 12C16O */
extern void CO_Init();
extern void CO_Stat();
extern void CO_Cik();

/* =========== in molco13.c ============ */
extern MOLECULE CO13;      /* 13C16O */
extern void CO13_Init();
extern void CO13_Stat();
extern void CO13_Cik();

/* =========== in molhco+.c ============ */
extern MOLECULE HCOp;   /* in molHCO+.c */
extern void HCOp_Init();
extern void HCOp_Stat();
extern void HCOp_Cik();

/* =========== in molsi2.c ============ */
extern MOLECULE SiII;   /* in molSiII.c */
extern void SiII_Init();
extern void SiII_Stat();
extern void SiII_Cik();

/* end of mol.h */
