/* molco13.c */
/* 92/11/15 New made from molco.c */
/* 93/10/28 definition of VMAX and JMAX are changed */
/*          they represent the number of level but not the maximum level */
/* 03/01/04 malloc.h -> stdlib.h */

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

#include "macro.h"
#include "const.h"
#include "funcs.h"
#include "matrix.h"

#include "mol.h"

#define i(v,j) ((v) * (JMAX) + (j))     /* array index */

#define VMAX  1               /* number of vibrational level */
#define JMAX  10              /* number of rotation level */
#define VJDIM (VMAX) * (JMAX)

#define MYU  0.1014           /* [debey] dipole moment */
#define B    (110.201353e9/C/2.0)     /* [cm-1] J=1 level energy */
#define EV   2074.81          /* [cm-1] v=1 level energy */

#define CDNF 1                /* collision excitation rate multiple factor */
                              /* used in CalcCik(), CDNF = 1 if mol. is CO */

MOLECULE CO13;

/* -------------------------------------------------------------- */
void CO13_Init(CO13)
   MOLECULE *CO13;
{
   int v, j, vl, vu, jl, ju;

   strcpy(CO13->name, "CO13");

   CO13->NV = VMAX;
   CO13->NJ = JMAX;
   CO13->LV = VJDIM;

/* memory allocation */
   CO13->g =    (int *)calloc((unsigned)VJDIM, (unsigned)sizeof(int));
   CO13->e =    (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));
   CO13->a =    d_alloc(VJDIM, VJDIM);
   CO13->de =   d_alloc(VJDIM, VJDIM);
   CO13->nyu0 = d_alloc(VJDIM, VJDIM);
   CO13->ct =   d_alloc(VJDIM, VJDIM);

   CO13->ng0 =  (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));
   CO13->ng =   (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));

/* ----- enargy level */
   for (v = 0; v < VMAX; v++)
      for (j = 0; j < JMAX; j++)
         CO13->e[i(v,j)] = EV * v + B * j * (j + 1);

/* ----- degeneracy */
   for (v = 0; v < VMAX; v++)
      for (j = 0; j < JMAX; j++)
         CO13->g[i(v,j)] = 2 * j + 1;

/* ----- Av,j<->v,j-1 */
   for (vl = 0; vl < VMAX; vl++)
      for (vu = vl; vu < VMAX; vu++)
         for (jl = 0; jl < JMAX; jl++)
            for (ju = jl; ju < JMAX; ju++)
   if (vl == vu)
      if (ju == jl + 1)
         CO13->a[i(vl,jl)][i(vu,ju)] = CO13->a[i(vu,ju)][i(vl,jl)]
            = 3.137E-7 * ju/(double)(2*ju+1) * cub(CO13->e[ju]-CO13->e[jl]) * sqr(MYU);
      else
         CO13->a[i(vl,jl)][i(vu,ju)] = CO13->a[i(vu,ju)][i(vl,jl)] = 0.0;
   else if (vu == vl + 1)
      if (ju == jl + 1)
         CO13->a[i(vl,jl)][i(vu,ju)] = CO13->a[i(vu,ju)][i(vl,jl)] =
         CO13->a[i(vu,jl)][i(vl,ju)] = CO13->a[i(vl,ju)][i(vu,jl)]
            = 34.4 * ju / (double)(2*ju+1);
      else
         CO13->a[i(vl,jl)][i(vu,ju)] = CO13->a[i(vu,ju)][i(vl,jl)] =
         CO13->a[i(vu,jl)][i(vl,ju)] = CO13->a[i(vl,ju)][i(vu,jl)] = 0.0;
   else
      CO13->a[i(vl,jl)][i(vu,ju)] = CO13->a[i(vu,ju)][i(vl,jl)] =
      CO13->a[i(vu,jl)][i(vl,ju)] = CO13->a[i(vl,ju)][i(vu,jl)] = 0.0;

/* ----- energy difference and frequency */
   for (vl = 0; vl < VMAX; vl++)
      for (vu = 0; vu < VMAX; vu++)
         for (jl = 0; jl < JMAX; jl++)
            for (ju = 0; ju < JMAX; ju++) {
               CO13->nyu0[i(vl,jl)][i(vu,ju)] = C*fabs(CO13->e[i(vu,ju)]-CO13->e[i(vl,jl)]);
               CO13->de[i(vl,jl)][i(vu,ju)] = -(CO13->e[i(vu,ju)]-CO13->e[i(vl,jl)])/WK;
            }

} /* CO13_Init */

/* -------------------------------------------------------------- */
void CO13_Stat(CO13, nH2, abn, tp)
   MOLECULE *CO13;
   double nH2, abn, tp;
{
#define QLIMIT 1.0E-15
   int v, j;
   double nsum, qj, f;
   static double eb, q, b;
   static double tsave = 0.0;

/* --- population --- */
   if (tp != tsave) {
      q = 0.0;
      j = 0;
      do {
         qj = 0.0;
         for (v = 0; v < VMAX; v++)
            qj = qj + (2*j+1) * exp(-(v*EV + B*j*(j+1))/WK/tp);
         q = q + qj;
         j++;
      } while (qj >= QLIMIT);
      tsave = tp;
   }
   CO13->nmol = nH2 * abn;
   nsum = 0;
   for (v = 0; v < VMAX; v++) {
      for (j = 0; j < JMAX; j++) {
         CO13->ng0[i(v,j)] = CO13->ng[i(v,j)] =
               CO13->nmol * exp(-CO13->e[i(v,j)] / WK / tp) / q;
         nsum = nsum + CO13->ng0[i(v,j)] * CO13->g[i(v,j)];
      }
   }
/*   nmol = nsum;*/

   CO13_Cik(CO13, tp, nH2);

}  /* CO13_Stat */

/* -------------------------------------------------------------- */
void CO13_Cik(CO13, tp, nH2)
   MOLECULE *CO13;
   double tp, nH2;
{
   int j, k;

   CO_Cik(CO13, tp, nH2);
   for (k = 0; k < JMAX; k++)
      for (j = k; j < JMAX; j++) {
         CO13->ct[j][k] *= CDNF;
         CO13->ct[k][j] *= CDNF;
      }
} /* CO13_Cik() */

/* end of molco13.c */
