/* molsiII.c */
/* Initialize for SiII ionic line */
/*    programed by I.Yamamura 93/10/11 based on molco.c */
/* 93/10/28 definition of VMAX and JMAX are changed */
/*          they represent the number of level but not the maximum level */
/* 03/01/04 malloc.h -> stdlib.h */

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

#include "macro.h"
#include "const.h"
#include "funcs.h"
#include "matrix.h"

#include "mol.h"

#define i(v,j) ((v) * (JMAX) + (j))     /* array index */

#define VMAX  1               /* number of vibrational level */
#define JMAX  10              /* number of rotation level */
#define VJDIM (VMAX) * (JMAX)

#define MYU  0.0              /* [debey] dipole moment */
#define B    287.240/2.0      /* [cm-1] J=1 level energy */
#define EV   0.0              /* [cm-1] v=1 level energy */

MOLECULE SiII;

/* -------------------------------------------------------------- */
void SiII_Init(SiII)
   MOLECULE *SiII;
{
   int v, j, vl, vu, jl, ju;

   strcpy(SiII->name, "SiII");

   SiII->NV = VMAX;
   SiII->NJ = JMAX;
   SiII->LV = VJDIM;

/* memory allocation */
   SiII->g =    (int *)calloc((unsigned)VJDIM, (unsigned)sizeof(int));
   SiII->e =    (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));
   SiII->a =    d_alloc(VJDIM, VJDIM);
   SiII->de =   d_alloc(VJDIM, VJDIM);
   SiII->nyu0 = d_alloc(VJDIM, VJDIM);
   SiII->ct =   d_alloc(VJDIM, VJDIM);

   SiII->ng0 =  (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));
   SiII->ng =   (double *)calloc((unsigned)VJDIM, (unsigned)sizeof(double));

/* ----- enargy level */
   for (v = 0; v < VMAX; v++)
      for (j = 0; j < JMAX; j++)
         SiII->e[i(v,j)] = EV * v + B * j * (j + 1);

/* ----- degeneracy */
   for (v = 0; v < VMAX; v++)
      for (j = 0; j < JMAX; j++)
         SiII->g[i(v,j)] = 2 * j + 1;

/* ----- Av,j<->v,j-1 */
   for (vl = 0; vl < VMAX; vl++)
      for (vu = vl; vu < VMAX; vu++)
         for (jl = 0; jl < JMAX; jl++)
            for (ju = jl; ju < JMAX; ju++)
         SiII->a[i(vl,jl)][i(vu,ju)] = SiII->a[i(vu,ju)][i(vl,jl)]
            = 2.17e-4;

/* ----- energy difference and frequency */
   for (vl = 0; vl < VMAX; vl++)
      for (vu = 0; vu < VMAX; vu++)
         for (jl = 0; jl < JMAX; jl++)
            for (ju = 0; ju < JMAX; ju++) {
               SiII->nyu0[i(vl,jl)][i(vu,ju)] = C*fabs(SiII->e[i(vu,ju)]-SiII->e[i(vl,jl)]);
               SiII->de[i(vl,jl)][i(vu,ju)] = -(SiII->e[i(vu,ju)]-SiII->e[i(vl,jl)])/WK;
            }

} /* SiII_Init */

/* -------------------------------------------------------------- */
void SiII_Stat(SiII, nH2, abn, tp)
   MOLECULE *SiII;
   double nH2, abn, tp;
{
#define QLIMIT 1.0E-15
   int v, j;
   double nsum, qj, f;
   static double eb, q, b;
   static double tsave = 0.0;

/* --- population --- */
   if (tp != tsave) {
      q = 0.0;
      j = 0;
      for (j = 0; j < JMAX; j++)
         q += (2*j+1) * exp(-(B*j*(j+1))/WK/tp);
      tsave = tp;
   }
   SiII->nmol = nH2 * abn;
   nsum = 0;
   for (v = 0; v < VMAX; v++) {
      for (j = 0; j < JMAX; j++) {
         SiII->ng0[i(v,j)] = SiII->ng[i(v,j)] =
               SiII->nmol * exp(-SiII->e[i(v,j)] / WK / tp) / q;
         nsum = nsum + SiII->ng0[i(v,j)] * SiII->g[i(v,j)];
      }
   }

/*   nmol = nsum;*/

   SiII_Cik(SiII, tp, nH2);

}  /* SiII_Stat */

/* -------------------------------------------------------------- */
void SiII_Cik(SiII, tp, nH2)
   MOLECULE *SiII;
   double tp, nH2;
{
   int j, k;

   for (k = 0; k < JMAX; k++)
      for (j = k; j < JMAX; j++) {
         SiII->ct[j][k] *= 0.0;
         SiII->ct[k][j] *= 0.0;
      }
} /* SiII_Cik() */

/* end of molSiII.c */
