
/* -------------------------------------------- */
void InitEfc2D(HPBW, sscl, SITE, efc, sumef)
/* make efficiency matrix for 2Dimension */
/* if HPBW == 0 then return flat beam 92/08/23 */
   double HPBW, sscl, **efc, *sumef;
   int SITE;
{
   int iy, iz;
   double e, r2, sgm2;

   *sumef = 0.0;
   if (HPBW == 0.0) {
      for (iy = 0; iy <= SITE; iy++)
         for (iz = 0; iz <= SITE; iz++) {
            efc[iy][iz] = 1.0;
            *sumef += 1.0;
         }
   } else {
      sgm2 = sqr(HPBW / sscl) / 8.0 / LN2;
      for (iy = 0; iy <= SITE; iy++) {
         for (iz = 0; iz <= SITE; iz++) {
            r2 = -(iy*iy + iz*iz) / 2.0 / sgm2;
            if (r2 < -50.0) r2 = -50.0;
            e = exp(r2);
            if (e < 1.0E-15) break;
            if ((iy <= SITE) && (iz <= SITE))
               efc[iy][iz] = e;
            if (iy != 0) e = e * 2.0;
            if (iz != 0) e = e * 2.0;
            *sumef += e;
/*            fprintf(stderr, "%3d %3d %e\n", iy, iz, e);/**/
         }
         if ((iz == 0) && (e < 1.0E-15)) break;
      }
   } /* if (HPBW) */
/*   printf("%e \n", *sumef); /**/

} /* InitEfc2D() */

/* -------------------------------------------- */
void InitEfcIF(HPBWy, HPBWz, theta, sscl, SITE, efc, sumef)
/* make beam pattern include elongation and inclination */
/* for interfelometers */
/* CAUTION that this routine needs 2*SITE+1 dimension for efc[] */
/* extended from InitEfc2D() 92/09/03 */
   double HPBWy, HPBWz, theta, sscl, **efc, *sumef;
   int SITE;
{
   int iy, iz, limit;
   double e, y2, z2, r2, sgm2y, sgm2z, sgm2t;

   theta = theta / 180.0 * PI;
   *sumef = 0.0;
   if (HPBWy*HPBWz == 0.0) {
      for (iy = 0; iy <= 2*SITE; iy++)
         for (iz = 0; iz <= 2*SITE; iz++) {
            efc[iy][iz] = 1.0;
            *sumef += 1.0;
         }
   } else {
      for (iy = 0; iy <= 2*SITE; iy++)
         for (iz = 0; iz <= 2*SITE; iz++)
               efc[iy][iz] = 0.0;
      *sumef = 0.0;
      sgm2t = sqr((HPBWy >= HPBWz ? HPBWy : HPBWz) / sscl) / 8.0 / LN2;
      for (limit = 0; exp(-sqr(limit) / 2.0 / sgm2t) > 1.0e-15; limit++);

      sgm2y = sqr(HPBWy / sscl) / 8.0 / LN2;
      sgm2z = sqr(HPBWz / sscl) / 8.0 / LN2;
      for (iy = -limit; iy <= limit; iy++) {
         for (iz = -limit; iz <= limit; iz++) {
            y2 = sqr(iy*cos(theta)-iz*sin(theta)) / 2.0/sgm2y;
            z2 = sqr(iy*sin(theta)+iz*cos(theta)) / 2.0/sgm2z;
            r2 = -(y2 + z2);
            if (r2 < -50.0) r2 = -50.0;
            e = exp(r2);
            if ((abs(iy) <= SITE) && (abs(iz) <= SITE))
               efc[iy+SITE][iz+SITE] = e;
            *sumef += e;
/*            printf("%3d %3d %e %e\n",iy,iz,e,*sumef);/**/
         }
      }
   } /* if (HPBW) */
/*   printf("%e \n", *sumef); /**/

} /* InitEfcIF() */

/* -------------------------------------------- */
double rEfc(r, sgm2)
/* calculate efficiency */
/* r and sgm2 must be in same unit */
   double r, sgm2;
{
   double ef;

   r = -sqr(r) / 2.0 / sgm2;
   if (r < -50.0) r = -50.0;
   ef = exp(r);

   return ef;

} /* rEfc() */
