/* model.c */
/* describe envelope model */
/* new made (proto-type) 93/08/20 I.Yamamura */
/* 03/01/04 malloc.h -> stdlib.h */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include "const.h"         /* physical constants */
#include "macro.h"         /* common macro functions */

#include "model.h"
#include "rt.h"

#define CONFIGFILE   "conf"


CONFIG cf;        /* here is the difinition of the variable */
MODEL md;         /* here is the definition of the variable */

/* ======================================================================= */
void ReadParam()
/* read parameter file */
{
   FILE *cnfn;
   char s[80], keywd[20];

   cnfn = fopen(CONFIGFILE, "r");
   if (cnfn == NULL) {
      fprintf(stderr, "ReadModelParam : configuration file %s not found.\n", CONFIGFILE);
      exit(-1);
   }
#define KWD(s, k) (strcmp((s), (k)) == 0)

   while (fgets(s, 80, cnfn) != NULL) {
      sscanf(s, "%s", keywd);
      if (s[0] == '#') ; /* comment */
/* configuration parameters */
      else if (KWD(keywd, "IMFLNM"))   sscanf(s, "%s%s",  keywd, cf.IMFLNM);
      else if (KWD(keywd, "SPFLNM"))   sscanf(s, "%s%s",  keywd, cf.SPFLNM);
      else if (KWD(keywd, "DISTANCE")) sscanf(s, "%s%le", keywd, &cf.DISTANCE);
      else if (KWD(keywd, "SNPIX"))    sscanf(s, "%s%d",  keywd, &cf.SNPIX);
      else if (KWD(keywd, "SRPIX"))    sscanf(s, "%s%d",  keywd, &cf.SRPIX);
      else if (KWD(keywd, "SRVAL"))    sscanf(s, "%s%le", keywd, &cf.SRVAL);
      else if (KWD(keywd, "SDELT"))    sscanf(s, "%s%le", keywd, &cf.SDELT);
      else if (KWD(keywd, "FNPIX"))    sscanf(s, "%s%d",  keywd, &cf.FNPIX);
      else if (KWD(keywd, "FRPIX"))    sscanf(s, "%s%d",  keywd, &cf.FRPIX);
      else if (KWD(keywd, "RESTFQ"))   sscanf(s, "%s%le", keywd, &cf.RESTFQ);
      else if (KWD(keywd, "VCENT"))    sscanf(s, "%s%le", keywd, &cf.VCENT);
      else if (KWD(keywd, "FWIDTH"))   sscanf(s, "%s%le", keywd, &cf.FWIDTH);
      else if (KWD(keywd, "HPBW"))     sscanf(s, "%s%le", keywd, &cf.HPBW);
      else if (KWD(keywd, "EFF"))      sscanf(s, "%s%le", keywd, &cf.EFF);
      else if (KWD(keywd, "HPBWY"))    sscanf(s, "%s%le", keywd, &cf.HPBWY);
      else if (KWD(keywd, "HPBWZ"))    sscanf(s, "%s%le", keywd, &cf.HPBWZ);
      else if (KWD(keywd, "PA"))       sscanf(s, "%s%le", keywd, &cf.PA);
      else if (KWD(keywd, "JU"))       sscanf(s, "%s%d",  keywd, &cf.JU);
      else if (KWD(keywd, "LTE_M"))    sscanf(s, "%s%d",  keywd, &cf.LTE_M);
      else if (KWD(keywd, "LVG_M"))    sscanf(s, "%s%d",  keywd, &cf.LVG_M);
      else if (KWD(keywd, "NOFF"))     sscanf(s, "%s%d",  keywd, &cf.NOFF);
      else if (KWD(keywd, "OFFSET1"))  sscanf(s, "%s%le", keywd, &cf.OFFSET1);
      else if (KWD(keywd, "OFFSET2"))  sscanf(s, "%s%le", keywd, &cf.OFFSET2);
      else if (KWD(keywd, "OFFSET3"))  sscanf(s, "%s%le", keywd, &cf.OFFSET3);
      else if (KWD(keywd, "OFFSET4"))  sscanf(s, "%s%le", keywd, &cf.OFFSET4);
/* model parameters */
      else if (KWD(keywd, "S_RO"))     sscanf(s, "%s%le", keywd, &md.S_RO);
      else if (KWD(keywd, "S_T"))      sscanf(s, "%s%le", keywd, &md.S_T);
      else if (KWD(keywd, "H_RI"))     sscanf(s, "%s%le", keywd, &md.H_RI);
      else if (KWD(keywd, "H_EI"))     sscanf(s, "%s%le", keywd, &md.H_EI);
      else if (KWD(keywd, "H_II"))     sscanf(s, "%s%le", keywd, &md.H_II);
      else if (KWD(keywd, "H_RO"))     sscanf(s, "%s%le", keywd, &md.H_RO);
      else if (KWD(keywd, "H_EO"))     sscanf(s, "%s%le", keywd, &md.H_EO);
      else if (KWD(keywd, "H_IO"))     sscanf(s, "%s%le", keywd, &md.H_IO);
      else if (KWD(keywd, "H_NE"))     sscanf(s, "%s%le", keywd, &md.H_NE);
      else if (KWD(keywd, "H_T"))      sscanf(s, "%s%le", keywd, &md.H_T) ;
      else if (KWD(keywd, "C_RI"))     sscanf(s, "%s%le", keywd, &md.C_RI);
      else if (KWD(keywd, "C_EI"))     sscanf(s, "%s%le", keywd, &md.C_EI);
      else if (KWD(keywd, "C_IC"))     sscanf(s, "%s%le", keywd, &md.C_IC);
      else if (KWD(keywd, "C_RO"))     sscanf(s, "%s%le", keywd, &md.C_RO);
      else if (KWD(keywd, "C_EO"))     sscanf(s, "%s%le", keywd, &md.C_EO);
      else if (KWD(keywd, "C_IO"))     sscanf(s, "%s%le", keywd, &md.C_IO);
      else if (KWD(keywd, "C_RB"))     sscanf(s, "%s%le", keywd, &md.C_RB);
      else if (KWD(keywd, "C_EB"))     sscanf(s, "%s%le", keywd, &md.C_EB);
      else if (KWD(keywd, "C_IB"))     sscanf(s, "%s%le", keywd, &md.C_IB);
      else if (KWD(keywd, "C_MLR"))    sscanf(s, "%s%le", keywd, &md.C_MLR);
      else if (KWD(keywd, "C_MOLNM"))  sscanf(s, "%s%s",  keywd,  md.C_MOLNM);
      else if (KWD(keywd, "C_ABN"))    sscanf(s, "%s%le", keywd, &md.C_ABN);
      else if (KWD(keywd, "C_T"))      sscanf(s, "%s%le", keywd, &md.C_T);
      else if (KWD(keywd, "C_VSYS"))   sscanf(s, "%s%le", keywd, &md.C_VSYS);
      else if (KWD(keywd, "C_VEXP"))   sscanf(s, "%s%le", keywd, &md.C_VEXP);
      else if (KWD(keywd, "C_TURB"))   sscanf(s, "%s%le", keywd, &md.C_TURB);
   }
} /* ReadParam() */

void PrintParam()
/* print parameters */
/* 93/08/25 I.Yamamura */
{
      printf("IMFLNM   = %s\n",  cf.IMFLNM);
      printf("SPFLNM   = %s\n",  cf.SPFLNM);
      printf("DISTANCE = %e\n",  cf.DISTANCE);
      printf("SNPIX    = %d\n",  cf.SNPIX);
      printf("SRPIX    = %d\n",  cf.SRPIX);
      printf("SRVAL    = %e\n",  cf.SRVAL);
      printf("SDELT    = %e\n",  cf.SDELT);
      printf("FNPIX    = %d\n",  cf.FNPIX);
      printf("FRPIX    = %d\n",  cf.FRPIX);
      printf("RESTFQ   = %e\n",  cf.RESTFQ);
      printf("VCENT    = %e\n",  cf.VCENT);
      printf("FWIDTH   = %e\n",  cf.FWIDTH);
      printf("HPBW     = %e\n",  cf.HPBW);
      printf("EFF      = %e\n",  cf.EFF);
      printf("HPBWY    = %e\n",  cf.HPBWY);
      printf("HPBWZ    = %e\n",  cf.HPBWZ);
      printf("PA       = %e\n",  cf.PA);
      printf("JU       = %d\n",  cf.JU);
      printf("LTE_M    = %d\n",  cf.LTE_M);
      printf("LVG_M    = %d\n",  cf.LVG_M);
      printf("NOFF     = %d\n",  cf.NOFF);
      printf("OFFSET1  = %e\n",  cf.OFFSET1);
      printf("OFFSET2  = %e\n",  cf.OFFSET2);
      printf("OFFSET3  = %e\n",  cf.OFFSET3);
      printf("OFFSET4  = %e\n",  cf.OFFSET4);
      printf("S_RO     = %e\n",  md.S_RO);
      printf("S_T      = %e\n",  md.S_T);
      printf("H_RI     = %e\n",  md.H_RI);
      printf("H_EI     = %e\n",  md.H_EI);
      printf("H_II     = %e\n",  md.H_II);
      printf("H_RO     = %e\n",  md.H_RO);
      printf("H_EO     = %e\n",  md.H_EO);
      printf("H_IO     = %e\n",  md.H_IO);
      printf("H_NE     = %e\n",  md.H_NE);
      printf("H_T      = %e\n",  md.H_T) ;
      printf("C_RI     = %e\n",  md.C_RI);
      printf("C_EI     = %e\n",  md.C_EI);
      printf("C_IC     = %e\n",  md.C_IC);
      printf("C_RO     = %e\n",  md.C_RO);
      printf("C_EO     = %e\n",  md.C_EO);
      printf("C_IO     = %e\n",  md.C_IO);
      printf("C_RB     = %e\n",  md.C_RB);
      printf("C_EB     = %e\n",  md.C_EB);
      printf("C_IB     = %e\n",  md.C_IB);
      printf("C_MLR    = %e\n",  md.C_MLR);
      printf("C_MOLNM  = %s\n",  md.C_MOLNM);
      printf("C_ABN    = %e\n",  md.C_ABN);
      printf("C_T      = %e\n",  md.C_T);
      printf("C_VSYS   = %e\n",  md.C_VSYS);
      printf("C_VEXP   = %e\n",  md.C_VEXP);
      printf("C_TURB   = %e\n",  md.C_TURB);
} /* PrintParam() */

/* ======================================================================= */
int Where(x, y, z)
/* return where is here */
   double x, y, z;         /* coordinate */
{
   double r = sqrt(sqr(x) + sqr(y) + sqr(z));

        if (r < md.S_RO)                  return CONT;
   else if (md.H_RI < r && r < md.H_RO)   return CONT;
   else if (md.C_RI < r && r < md.C_RO)   return LINE;
   else                                   return CONT;
} /* Where() */

/* ======================================================================= */
void MolEnv(x, y, z, nH2, tk, abn, v, turb, dvdr)
/* describe physical condition of molecular envelope */
   double x, y, z;         /* position */
   double *nH2;            /* H2 density */
   double *tk;             /* kinetic temperature */
   double *abn;            /* abundance of the molecule */
   double *v;              /* radial velocity */
   double *turb;           /* turblance velocity */
   double *dvdr;           /* dv/dr */
{
   double r, theta, phai;
   double v2, dv;
   double vth2;
   double X, Y, Z;
   double si, c, f;

   static double r0;
   static double nH2_0;
   static int FIRST = 0;
   static double cs75, cs88;

#define DR 1.001
#define Vr(r) md.C_VEXP /**/
#define THA (45.0 / 180.0 * PI)
#define R75 (75.0 / 180.0 * PI)
#define R88 (88.0 / 180.0 * PI)

   if (FIRST == 0) {
      r0 = md.C_RI;
      nH2_0 = md.C_MLR * MSUN/(4.0 * PI * sqr(r0) * md.C_VEXP * ONE_YR * MH2);
      cs75 = cos(R75);
      cs88 = cos(R88);
      FIRST = -1;
   }

   Rotate(x, y, &Z, &Y, THA);
   X = z;

   Car2Sph(X, Y, Z, &r, &theta, &phai);

   theta = fabs(theta - PI / 2.0);

   if (theta < R88)
      c = 1.0 / (1.0 + sqr((cs75 - cs88) / (cos(theta) - cs88)));
   else
      c = 1.0e-5;
   si = sin(theta);
   f = (1.0 - si + 0.5 * sqr(si) - 1.0 / 6.0 * cub(si)) * c;

/* density and abundance */
   if (md.C_RB > 0.0 && r < md.C_RB) {
      *nH2 = nH2_0 * sqr(r0 / r) * f;
   } else {
      *nH2 = nH2_0 * sqr(r0 / r);
   }

   *abn = md.C_ABN;

/* temperature */
   *tk = 60.0 * pow(r / 1.0e16, -0.477);
/*   *tk = md.C_T;*/

/* velocity structure */
   *v = Vr(r);
   if (fabs(*v) < 1.0E-3) *v = sgn(*v) * 1.0E-3;
   v2 = Vr(r * DR);
   dv = (v2 - *v);
   vth2 = (8 * BK * (*tk) / PI / MH2);       /* Vth^2 */
   *turb = sqrt(vth2 + sqr(md.C_TURB));
   *dvdr = sqrt(sqr(dv / (r * (DR - 1.0))) + (vth2 + sqr(md.C_TURB))/sqr(*v));

} /* MolEnv() */

/* ======================================================================= */
void ContEnv(x, y, z, te, ne)
/* describe physical condition of continuum region */
/* 93/08/22 I.Yamamura */
/* 93/08/25 return electron density instead of optical depth, */
/*          so that input parameter nyu is not necessary */
   double x, y, z;         /* position */
   double *te;             /* electron temperature (K) */
   double *ne;             /* electron density (cm-3) */
{
   double r = sqrt(sqr(x) + sqr(y) + sqr(z));

   if (md.H_RI <= r && r <= md.H_RO) {
      *te = md.H_T;
      *ne = md.H_NE;
   } else {
      *te = 0.0;
      *ne = 0.0;
   }      
} /* ContEnv() */

/* ================== to access phyical parameters =================== */

double R_Star()
/* Radius of the central star (or IR-region) */
{
   return md.S_RO;
} /* R_Star() */

double T_Star()
/* Temperature of the central star (or IR-region) */
{
   return md.S_T;
} /* T_Star() */

double RO_HII()
/* (effective) outer radius of the HII region */
{
   return md.H_RO * (1 + md.H_EO) / 2.0;
} /* RO_HII() */

/* model.c */
